/**
 * Format.java
 *
 * Java Bro Fuzzer. A stateless network protocol fuzzer for penetration tests.
 * It allows for the identification of certain classes of security bugs, by
 * means of creating malformed data and having the network protocol in question
 * consume the data.
 *
 * Copyright (C) 2006 subere (at) uncon . org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */
package jbrofuzz;

/**
 * <p>Description: Class responsible for holding the description of a number of
 * parameters and constants relating to the application.</p>
 *
 * @author subere (at) uncon . org
 * @version 0.3
 */
public class Format {
  /**
   * <p>The header used when writting a single fuzz request.</p>
   */
  public static final String HD1 =
      "\n-----JBroFuzz------Start--(1 of 1)------\n";
  /**
   * <p>The footer used when writting a single fuzz request.</p>
   */
  public static final String FT1 =
      "\n-----JBroFuzz------End----(1 of 1)------\n";
  /**
   * <p>The header used when writting any fuzz request (other than a single
   * request).</p>
   */
  public static final String HDS = "\n-----JBroFuzz------Start--(";
  /**
   * <p>The footer used when writting any fuzz request (other than a single
   * request).</p>
   */
  public static final String HDE = "\n-----JBroFuzz------End----(";
  /**
   * <p>The end of the String used to complete both HDS and HDE Strings.</p>
   */
  public static final String FTS = ")------\n";
  /**
   * <p>The system info providing console information and potentially useful
   * for debugging under different JVMs.</p>
   *
   * @since 0.1
   */
  public static final String SYSTEM_INFO =
      "Vendor:  " + System.getProperty("java.vendor") + "\r\n" +
      "Version: " + System.getProperty("java.version") + "\r\n" +
      "Installed at: " + System.getProperty("java.home") + "\r\n" +
      "Website: " + System.getProperty("java.vendor.url") + "\r\n" +
      "User: " + System.getProperty("user.name") + "\r\n" +
      "Home $: " + System.getProperty("user.home") + "\r\n" +
      "Current $: " + System.getProperty("user.dir") + "\r\n" +
      "O/S: " + System.getProperty("os.name") + "\r\n" +
      "Version: " + System.getProperty("os.version") + "\r\n" +
      "Architecture: " + System.getProperty("os.arch") + "\r\n";

/**
 * <p>The list of generators used if a file is not found to load the generators
 * from.</p>
 */
public static final String DEFAULT_GENERATORS =
    "\nR:HEX:Hexadecimal Fuzz Type:16\n" +
    "> Category String | Characters | Hex\n" +
    "0\n1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf\n\n" +
    "R:DEC:Decimal Fuzz Type:10\n" +
    "> Category String | Characters | Dec\n" +
    "0\n1\n2\n3\n4\n5\n6\n7\n8\n9\n\n" +
    "R:OCT:Octal Fuzz Type:8\n" +
    "> Category String | Characters | Octal\n" +
    "0\n1\n2\n3\n4\n5\n6\n7\n\n" +
    "R:BIN:Binary Fuzz Type:2\n" +
    "> Category String | Characters | Binary\n" +
    "0\n1\n";
}
